from typing import Optional
from venv import create

import typer

from wombopy import __app_name__, __version__, create, identify
from wombopy.logging import wombolog
from wombopy.core.generation import show_img

app = typer.Typer()


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()


@app.command(help="Create an image from wombo.art")
def run(
    open_result: Optional[bool] = typer.Option(
        False, "--show", "-s", help="Show back the image."
    ),
    debug: Optional[bool] = typer.Option(
        True, "--debug", help="Show logs."
    ),
    identify_key: str = typer.Argument(...),
    prompt: str = typer.Argument(...),
    style: int = typer.Argument(...),
) -> None:
    wombolog.propagate = debug
    
    wombolog.info(f"Prompt: {prompt}\tStyle: {style}")
    wombolog.info("Starting...")

    res = identify(identify_key=identify_key)
    img_uri = create(res["id_token"], prompt, style)
    
    if open_result:
        show_img(img_uri)

    return


@app.callback()
def main(
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        help="Show the application's version and exit.",
        callback=_version_callback,
        is_eager=True,
    )
) -> None:
    return



import requests

from wombopy.logging import wombolog


def sign_up(key):
    body = {"key": key}

    r = requests.post(
        f"https://identitytoolkit.googleapis.com/v1/accounts:signUp?key={key}",
        data=body,
    )

    if r.status_code != requests.codes.ok:
        error = f"Error during identification. Status code error: {r.status_code}"
        wombolog.info(error, err=True)
        assert False, error

    wombolog.info("Google identification sign up.")
    id_token = r.json()["idToken"]
    wombolog.info("  => idToken got.")
    return id_token


def look_up(identify_key, id_token):
    body = {"idToken": id_token}

    r = requests.post(
        f"https://identitytoolkit.googleapis.com/v1/accounts:lookup?key={identify_key}",
        data=body,
    )

    if r.status_code != requests.codes.ok:
        assert False, f"Error during identification. Status code error: {r.status_code}"

    wombolog.info("Google identification look up.")
    local_id = r.json()["users"][0]["localId"]
    wombolog.info("  => localId got.")
    return local_id


def identify(identify_key):
    id_token = sign_up(identify_key)
    local_id = look_up(identify_key, id_token)
    wombolog.info("Identification done!")

    return {"id_token": id_token, "local_id": local_id}



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wombopy",
    version="0.2.2",
    description="A module to generate wombo.art images",
    author="Holy Tanuki",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "typer==0.7.0",
        "colorama==0.4.6",
        "shellingham==1.5.0post1",
        "pytest==7.1.2",
        "requests==2.28.2"
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.7.13",
)



import json
import time
from io import StringIO, BytesIO

import requests

from wombopy.logging import wombolog


def task(session, id):
    r = session.get(f"https://paint.api.wombo.ai/api/tasks/{id}")

    rep = r.json()
    wombolog.info(f"Status: {rep['state']}")
    return rep


def show_img(url):
    try:
        from PIL import Image
    except ImportError:
        raise ImportError("Pillow image is needed to use this function.")

    r = requests.get(url)

    with Image.open(BytesIO(r.content)) as im:
        im.show()

        return im


def create(id_token: str, prompt: str, style: int):
    s = requests.Session()
    s.headers.update(
        {
            "Authorization": "bearer " + id_token,
            "Origin": "https://paint.api.wombo.ai/",
            "Referer": "https://paint.api.wombo.ai/",
            "User-Agent": "Mozilla/5.0",
        }
    )

    def init_task():
        body = StringIO()
        json.dump({"premium": False}, body)

        r = s.post("https://paint.api.wombo.ai/api/tasks", data=body.getvalue())

        return r.json()["id"]

    id = init_task()
    
    display_freq = 10
    
    body = {
        "input_spec": {
            "prompt": prompt,
            "style": style,
            "display_freq": display_freq,
        },
        "is_premium": False
    }

    body = json.dumps(body)
    print(body)
    r = s.put(f"https://paint.api.wombo.ai/api/tasks/{id}", data=body)
    
    print(r.json())

    wombolog.info(f"Status: {r.json()['state']}")
    display_freq = display_freq / 10

    latest_task = task(s, id)
    while latest_task["state"] != "completed":
        time.sleep(display_freq)
        latest_task = task(s, id)

    result = s.post(f"https://paint.api.wombo.ai/api/tradingcard/{id}")
    img_uri = result.json()

    if img_uri:
        wombolog.info(f"Url result: {img_uri}")
        return img_uri
    else:
        wombolog.info("Invalid image uri, can't download result!")
